package com.ynxbd.push.httpRequest.xbd.wx;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ynxbd.push.config.EhCacheConfig;
import com.ynxbd.push.config.IniConfig;
import com.ynxbd.push.entity.xbd.wx.AccessToken;
import com.ynxbd.push.helper.DateHelper;
import com.ynxbd.push.helper.DesEncryptHelper;
import com.ynxbd.push.helper.RequestHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.ehcache.Cache;

@Slf4j
public class WxCacheRequest {

    private static final int MAX_RETRY = 3;

    static {
        createAccessTokenCache();
    }

    // 缓存

    private static Cache<String, AccessToken> ACCESS_TOKEN_CACHE;

    private static final String CACHE_NAME = "access_token";
    private static final long CACHE_TIME = 7100;


    private synchronized static void createAccessTokenCache() {
        if (ACCESS_TOKEN_CACHE == null) {
            // 一个半小时
            ACCESS_TOKEN_CACHE = EhCacheConfig.createCacheTTL(String.class, AccessToken.class, "wx_access_token_cache", (CACHE_TIME));
        }
    }

    public static String getAccessToken() {
        AccessToken accessToken = getWxAccessToken();
        if (accessToken == null) {
            return null;
        }
        return accessToken.getAccessToken();
    }

    /**
     * 获取微信token
     */
    private synchronized static AccessToken getWxAccessToken() {
        if (ACCESS_TOKEN_CACHE == null) {
            createAccessTokenCache();
        }
        try {
            AccessToken token = new AccessToken();
            AccessToken cacheToken = ACCESS_TOKEN_CACHE.get(CACHE_NAME);
            if (cacheToken != null) {
                // 允许100s的旧token存在（5分钟内可以继续使用佬）
                if (7200 < DateHelper.differCurSecond(cacheToken.getCreateTime())) {
                    AccessToken respToken = againGetWxAccessToken();
                    if (respToken != null) {
                        token.setAccessToken(respToken.getAccessToken());
                        token.setCreateTime(respToken.getCreateTime());
                        ACCESS_TOKEN_CACHE.put(CACHE_NAME, respToken);
                    }
                } else {
                    token.setAccessToken(cacheToken.getAccessToken());
                    token.setCreateTime(cacheToken.getCreateTime());
                }
                return token;
            }

            AccessToken respToken = againGetWxAccessToken();
            if (respToken != null) {
                token.setAccessToken(respToken.getAccessToken());
                token.setCreateTime(respToken.getCreateTime());
                ACCESS_TOKEN_CACHE.put(CACHE_NAME, respToken);
            }
            return token;
        } catch (Exception e) {
            log.error("请求微信获取Token错误:{}", e.getMessage());
        }
        return null;
    }

    /**
     * 根据次数循环获取
     */
    private static AccessToken againGetWxAccessToken() {
        AccessToken accessToken;
        for (int i = 0; i < MAX_RETRY; i++) {
            accessToken = requestWxAccessToken();
            if (i != 0) {
                log.error("重试请求第{}次", (i + 1));
            }
            if (accessToken != null) {
                return accessToken;
            }
            if ((i + 1) == MAX_RETRY) {
                log.error("{} 次请求后未获取到正确token", MAX_RETRY);
            }
        }
        return null;
    }


    /**
     * 请求微信token
     */
    private static AccessToken requestWxAccessToken() {
        try {
            String token_url = StringUtils.isNotBlank(IniConfig.TOKEN_URL)?IniConfig.TOKEN_URL:"http://127.0.0.1:8080/wx/api/getWxAccessToken.do";
            JSONObject respJson = RequestHelper.getJsonObj(token_url, null);
            String code = respJson.getString("code");
            if (!"200".equals(code)) {
                log.info("请求token失败 code={}, message={}", code, respJson.getString("message"));
                return null;
            }
            String data = respJson.getString("data");
            if (data == null || "".equals(data)) {
                return null;
            }
            AccessToken resp = JSON.parseObject(data, AccessToken.class);
            System.out.println(resp);
            if (resp == null) {
                return null;
            }
            String accessToken = resp.getAccessToken();
            if (accessToken == null) {
                return null;
            }
            accessToken = DesEncryptHelper.deCode(accessToken);
            if (accessToken == null) {
                return null;
            }
            resp.setAccessToken(accessToken);
            return resp;
        } catch (Exception e) {
            log.error("请求微信获取Token错误:{}", e.getMessage());
        }
        return null;
    }

    public synchronized static void clearWxAccessToken() {
        if (ACCESS_TOKEN_CACHE != null) {
            ACCESS_TOKEN_CACHE.remove(CACHE_NAME);
        }
    }

}
